// -*- mode: c++ -*-
// Copyright (C) 2002 mediaWays GmbH Internet-Services, Verl, Germany
//
// PURPOSE OF THIS FILE: Implement the ldap_result functions
//
// - Automatic Version Information via RCS:
//   $Id: ldapgetresults.cxx,v 1.1 2002/09/23 07:28:52 robertj Exp $
//   $Source: /home/cvsroot/openh323/src/ldapgetresults.cxx,v $

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * The Original Code is the OpenLDAP Comatible LDAP-API for OpenH323
 * (libldapapi).
 *
 * The Initial Developer of the Original Code is
 * mediaWays GmbH Internet-Services, Verl, Germany
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Nils Bokermann <Nils.Bokermann@mediaWays.net> (*)
 *                 Martin Frhlich <Martin.Froehlich@mediaWays.net> (*)
 *
 * (*) Contributors marked this way are mediaWays employees, all their work
 *     is subject to mediaWays copyright by their contracts.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * For copies of the GPL or LGPL, see either http://www.fsf.org/licenses/
 * or write to the Free Software Foundation, Inc., 59 Temple Place - Suite
 * 330, Boston, MA 02111-1307, USA.
 *
 * ***** END LICENSE BLOCK ***** */

#include <ptlib.h>
#include <ldapapi.h>
#include "ldap-int.h"


#ifndef lint
// mark object with version info in such a way that it is retrievable by
// the std. version/revision control tools like RCS/CVS ident cmd.. At
// least the strings cmd will extract this info.
volatile static const char vcid[] = "@(#) $Id: ldapgetresults.cxx,v 1.1 2002/09/23 07:28:52 robertj Exp $";
volatile static const char vcHid[] = LDAP_OPENH323_H;
#endif

LDAPMessage * ldap_first_entry (LDAP *ld, LDAPMessage *entry) {
  if( ld == NULL || entry == NULL ) {
    return NULL;
  }
  if((&(entry->message->m_protocolOp.GetObject()))->IsClass("LDAP_SearchResponse")) {
    LDAP_SearchResponse &res=entry->message->m_protocolOp;
    return res.GetTag()==LDAP_SearchResponse::e_entry ? entry  : ldap_next_entry(ld, entry);
  }
  return entry;
}


LDAPMessage * ldap_next_entry (LDAP *ld, LDAPMessage *entry) {
  if ( ld == NULL || entry == NULL ) {
    return NULL;
  }

  for (entry = entry->next; entry != NULL; entry = entry->next) {
    if((&(entry->message->m_protocolOp.GetObject()))->IsClass("LDAP_SearchResponse")) {
      LDAP_SearchResponse &res=entry->message->m_protocolOp;
      if (res.GetTag()==LDAP_SearchResponse::e_entry)
	return entry;
    }
  }
  return(NULL);
}

int ldap_count_entries (LDAP *ld, LDAPMessage *entry){
  int	i;

  if ((ld == NULL) || (NULL==entry))
    return -1;
  
  for (i=0; entry!=NULL; entry=entry->next) {
    if((&(entry->message->m_protocolOp.GetObject()))->IsClass("LDAP_SearchResponse")) {
      LDAP_SearchResponse &res=entry->message->m_protocolOp;
      if (res.GetTag()==LDAP_SearchResponse::e_entry)
	i++;
    }
  }
  return( i );
}

//
// End of getresults.cxx
//
